import re
from typing import Dict, Iterator, List, Optional

import typer
from bs4 import BeautifulSoup

from .date_case import date_validator
from .split import use_citation_to_divide
from .utils import (
    RE_PATTERN,
    VS_PATTERN,
    get_indicators,
    match_any_versus,
    ok_count_of_vs_variants_in_tags,
)
from .validate_case import case_italics_validator

app = typer.Typer()


def make_candidates(
    source_text: str,
    footnote_index: Optional[int] = None,
) -> Optional[List[Dict]]:
    """If footnote_index exists, text is a footnote value; otherwise it's a paragraph segment of the decision"""
    if not get_indicators(source_text):
        return None
    items: Iterator = collect_candidates(source_text, footnote_index)
    candidates: List[Dict] = sum(list(items), [])  # must unflatten to check
    return candidates or None


def collect_candidates(
    text: str, key: Optional[int] = None
) -> Iterator[List[Dict]]:
    """
    1. Create lines from the text
    2. Lines are text split by ; and/or delimiter keywords
    3. Look for casenames in each line
    4. Yield casenames into iterator of lists of dicts
    """
    for textline in divided_caselines(text):

        # convert text to soup
        soup = BeautifulSoup(textline, "html5lib")

        # matches <em> vs </em> variants
        adverse_style = soup("em", text=VS_PATTERN)

        # matches <em> In re: </em> variants
        inre_style = soup("em", text=RE_PATTERN)

        # are there possible style matches?
        if (styles := adverse_style or inre_style) :

            # e.g. X <em>v.</em> Y; edge case priority
            if ok_count_of_vs_variants_in_tags(styles) and (
                data := list(use_citation_to_divide(textline, text, key))
            ):
                yield data

            # e.g. <em> X v. Y GR NO</em> DATE, edge case priority
            elif (date_case := date_validator(styles[0], textline, key)) :
                yield [date_case]

            # e.g. <em> X v. Y</em>, <em> In Re: X</em>
            elif (data := list(case_italics_validator(styles, text, key))) :
                yield data

        # e.g. X v. Y
        elif (data := list(use_citation_to_divide(textline, text, key))) :
            yield data


def divided_caselines(text: str) -> Iterator[str]:
    """
    Multiple case names can be found in either a given footnote text or in a paragraph of the body of a decision. Split lines of the source text, these lines will likely contain vs tags. With the given text, split into candidate lines that possibly contain a case. Splits can be based on delimiters such as (1.) a semi-colon ';' which comes in between cases (2.) the words "see also", "citing", "cited" (3.) variants of "supra"
    >>> list(divided_caselines("Golangco <em>v.</em> Villanueva, 278 SCRA 414, 422-423 (1997). Reiterated in Reynaldo V. Abundo <em>v.</em> Judge Gregorio E. Manio, Jr., A.M. No. RTJ-98-1416, Aug. 6, 1999."))
    ['Golangco <em>v.</em> Villanueva, 278 SCRA 414, 422-423 (1997). Reiterated in',
    'Reynaldo V. Abundo <em>v.</em> Judge Gregorio E. Manio, Jr., A.M. No. RTJ-98-1416, Aug. 6, 1999.']
    """
    delimiters = [
        r"(?<!&amp);",  # negative lookbehind must split by ";" see "<em>People vs. Matrimonio,</em> supra Fn. 33; People vs.Echegaray, G.R. No. 117472, June 25, 1996, 257 SCRA 561." but not by "&amp;" "Reynes v. Barrera,</em> 68 Phil. 656, 658; <em>W.H. Anderson &amp; Co. v. Garcia</em>, 64 Phil. 506, 515;"
        r"cit(ed|ing)\b",  # People <em>v.</em> Lucanas, <em>supra</em> note 21, citing People <em>v.</em> Vitor, 245 SCRA 392 (1995).
        r"quoting\b",
        r"\bSee\s+also\s+(?=[A-Z])",
        r"\bSee\s+(?=[A-Z])",
        r"(^|\s)SEE\s+(?=[A-Z])",
        r"\(See",
        r",\s*supra,\s*",
        r"<em>supra</em>",
        r"(Q|q)uoted\s+with\s+approval\s+in",
        r"(R|r)eiterated in",
    ]
    patterns = re.compile("|".join(delimiters))
    while True:
        if (match := patterns.search(text)) :
            index = match.end() if match else len(text)
            item = text[:index]
            text = text.removeprefix(item).strip()
            if match_any_versus(item):
                yield item.strip()
        else:
            yield text
            break
