from typing import Dict, Iterator, List, Optional

from bs4 import NavigableString, Tag
from casecite import first_docket_id, make_citations
from dateutil.parser import parse

from .split import create_single_title


def is_date(string, fuzzy=False):
    """ Return whether the string can be interpreted as a date. """
    try:
        parse(string, fuzzy=fuzzy)
        return True

    except:
        return False


def date_validator(
    focus: Tag, source: str, key: Optional[int] = None
) -> Optional[Dict]:
    """Get casename matching criteria: <em>GR NUM</em>DATE;
    should solve <em>People v. De los Reyes</em>, G.R. No. 140680, 28 May 2004, 430 SCRA 166, 173; <em>Senoja v. People, G.R. No. 160341, </em>19 October 2004, 440 SCRA 695, 706.
    should solve <em>Ferrer v. People, G.R. No. 143487, </em>22 February 2006, 483 SCRA 31, 54.
    """

    # does the focus have content?
    if not focus or not focus.get_text():
        return None

    # is neighbor of focus a date?
    if not (
        focus.next_sibling
        and isinstance(focus.next_sibling, NavigableString)
        and is_date(focus.next_sibling, fuzzy=True)
    ):
        return None

    # combine neighboring tags and detect passable citation
    text = focus.get_text() + focus.next_sibling
    if not (citations := make_citations(text)):
        return None

    # existence of the raw key implies a footnote
    if not (raw_citation := citations[0].get("raw", None)):
        return None

    # citation must be a docket id since the neighbor is a date
    if not (docket_found := first_docket_id(raw_citation)):
        return None

    # test if the criteria is met
    if docket_found not in focus.get_text():
        return None

    # is there a single candidate in the line / source
    if not (item := create_single_title(text, source, key)):
        return None

    return item
