import re


def missing_brackets(text: str) -> str:
    # general cleaning
    if "(" not in text:
        text = text.rstrip(")")
    if ")" not in text:
        text = text.lstrip("(")
    if "[" not in text:
        text = text.rstrip("]")
    if "]" not in text:
        text = text.lstrip("[")
    return text


def uniform_replacements(text: str) -> str:
    if text in [
        "US",
        "U.S.",
        "US.",
        "U.S",
    ]:
        text = "United States"
    elif text in [
        "People",
        "PP",
        "Pp.",
        "eople",
    ]:
        text = "People of the Philippines"
    elif text in [
        "CA",
        "C.A.",
    ]:
        text = "Court of Appeals"
    elif text in [
        "IAC",
        "I.A.C.",
    ]:
        text = "Intermediate Appellate Court"
    elif text in [
        "CSC",
    ]:
        text = "Civil Service Commission"
    elif text in [
        "ECC",
        "Employees’ Compensation Commission",
        "Employees' Compensation Commission",
    ]:
        text = "Employees Compensation Commission"
    elif text in [
        "GSIS",
    ]:
        text = "Government Service Insurance System"
    elif text in [
        "NLRC",
        "Nlrc",
    ]:
        text = "National Labor Relations Commission"
    elif text in [
        "PNB",
        "Pnb",
    ]:
        text = "Philippine National Bank"
    elif text in ["COMELEC", "Comelec"]:
        text = "Commission on Elections"
    elif text in ["COA"]:
        text = "Commission on Audit"
    elif text in ["SEC"]:
        text = "Securities and Exchange Commission"
    elif text in ["CIR", "C.I.R."]:
        text = "Commissioner of Internal Revenue"
    return text


def remove_infra(text: str) -> str:
    pattern = re.compile(
        r"""
        \,?
        \s*
        \(?
        [Ii]nfra
        \)? # (infra).
        \s*
        (note)?
        $
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_supra(text: str) -> str:
    pattern = re.compile(
        r"""
        \,?
        \s*
        \(?
        [Ss]upra
        \)? # (supra).
        \s*
        (
            (
                [\.,:;\s]+ # supra. | supra, | supra: | supra.;
            )|
            ( # , supra note [number] | supra note [number] | , Supra note [number] | , supra, note [number] | , supra, Note [number]
                ,?
                \s*
                [Nn]ote
                \s*
                \d+
                [\.,;]?
                (
                    (
                        \s*
                        at
                        \s*
                        [\d-]+
                    )|
                    ( # NLRC, supra note 23, pp. 192-193
                        \s*
                        pp?
                        \.?
                        \s*
                        [\d-]+
                    )
                )?
            )|
            ( #, supra, p. [number] | supra, p. [number]-[number] | supra, pp. [number]-[number]
                ,
                \s*
                pp?
                \.?
                \s*
                [\d-]+
            )|
            ( # , supra at [number]
                ,?
                \s*
                at
                \s*
                [\d-]+
            )
        )?
        [\.,;]?
        $ # end of line

        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_case(text: str) -> str:
    return text.removesuffix(" case")


def remove_prefixes(text: str) -> str:
    return (
        text.removeprefix("and ")
        or text.removeprefix("and, ")
        or text.removeprefix("see also ")
    )


def remove_eg(text: str) -> str:
    pattern = re.compile(
        r"""
        ^
        (E|e)\.
        g\.
        \,
        \s+
        (?= # positive lookahead for a capital letter
            [A-Z]
        )
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_in(text: str) -> str:
    """
    e.g. "Court ruled in Interpacific Transit, Inc." --> signal is "in"
    """
    pattern = re.compile(
        r"""
        ^
        .*
        \b
        (in|In)
        \s+
        (?= # positive lookahead for a capital letter
            [A-Z]
        )
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_case_of(text: str) -> str:
    """
    e.g. "the recent case of Lozon v. NLRC" --> signal is "case of"
    """
    pattern = re.compile(
        r"""
        .*
        \b
        case
        \s+
        of
        \s+
        (?= # positive lookahead for a capital letter
            [A-Z]
        )
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_cf(text: str) -> str:
    """
    e.g. "Cf. People vs. Magpale,"
    """
    pattern = re.compile(
        r"""
        .*
        \b
        (Cf|cf)
        \.?
        \s+
        (?= # positive lookahead for a capital letter
            [A-Z]
        )
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_citing(text: str) -> str:
    """
    e.g. "citing National Brewery and Allied Industries Labor Union of the Philippines v. San Miguel Brewery, Inc.,"
    e.g. "see Butuan City vs. Ortiz, et. al.,"
    """
    pattern = re.compile(
        r"""
        .*
        \b
        (Citing|citing|cites|See|see)
        \s+
        (?= # positive lookahead for a capital letter
            [A-Z]
        )
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_etal(text: str) -> str:
    pattern = re.compile(
        r"""
        \,?
        \s*
        et
        \.?
        \s+
        a[l1] # See et a1 in 36209
        \.?
        \s*
        (
            \,| # et al.,
            \:| # et al.:
            \"| # et al."
            \"\,| # et al.",
            \,\,| # et al.,,
            \,\" # et al.,"
        )?
        $ # end of line
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text


def remove_etc(text: str) -> str:
    pattern = re.compile(
        r"""
        \,?
        \s*
        etc
        \.?
        \,?
        $ # end of line
        """,
        re.X,
    )
    return pattern.sub("", text) if pattern.search(text) else text
