import re
from typing import Dict, List, Optional, Tuple

from bs4 import BeautifulSoup
from casecite import make_citations

from .utils import one_case_pattern


def create_single_title(
    target: str,
    source: str,
    key: Optional[int] = None,
) -> Optional[Dict]:
    """ Is there a single case-like item found in the text? """
    # there should only be one case in the target
    if not one_case_pattern(target):
        return None

    # the source parameter becomes the "raw" key of the dictionary
    item = {
        "raw": source if key else None,
        "key": key,
        "casename": None,
        "citation": None,
        "plaintiff": None,
        "defendant": None,
        "inre": None,
        "error": None,
    }
    item["casename"], item["citation"] = cullcite(target)
    return item


def cullcite(text: str) -> Tuple[str, Optional[str]]:
    """ Does text contain a citation; modify the same. """
    if not (citations := make_citations(text)):
        return text, None

    citation = re.escape(citations[0]["raw"])
    # NOTE: pattern may contain arbitrary regex literals like (,
    # this needs to be escaped, otherwise will result in
    # re.error: missing ), unterminated subpattern at position ...

    citation_removed = re.sub(citation, "", text)

    return citation_removed, citations[0]["raw"]


def get_candidates(text: str) -> List[str]:
    """
    Presumes that text has been converted to raw string, e.g.
    <em> X v. Y; A v. B </em> is now just "X v. Y; A v. B"

    Although covered by a single em tag, there are two valid casenames; # e.g. <em>X v. Y (citation); A v. B (citation)</em>
    likeso: <em>People vs. Querido,</em> G.R. No. 95319, February 7, l994, 229 SCRA 745; <em>People vs. Israel,</em> G.R. No. 97027, March 11, l994; 231 SCRA 155; <em>People vs. Silong, et al.,</em> G.R. No. 110830, May 23, l994, 232 SCRA 487.
    If there are no ';', only one candidate appears. Absence of ';' does not mean only one case; must use citation as splitting mechanism
    in cases similar to "<em>People vs. Villagracia, 226 SCRA 374, 381 citing case of Sacay vs. Sandiganbayan,</em> 142 SCRA 593 (1986).

    # Does not work for formats like:
    # People <em>v.</em> Lucanas, <em>supra</em> note 21, citing People <em>v.</em> Vitor, 245 SCRA 392 (1995).
    # Here 'supra' as the citation is not split after a case name
    # So must split before getting candidates
    """
    candidates = []
    for subtext in text.split(";"):
        candidates += list(separate_citations(subtext))
    return candidates


def separate_citations(target_text: str):
    """ Reduce the text in every iteration of a while loop """
    while True:
        if (cites := make_citations(target_text)) :
            # get first citation found in the text
            cite = cites[0]["raw"]
            # enable escape characters
            cite_escaped = re.escape(cite)
            # get a match object for the citation
            match = re.search(cite_escaped, target_text)
            # get the index of the matching text, if applicable
            end_span = match.end() if match else len(target_text)
            # consider the sliced text to be an item
            item = target_text[:end_span]
            # the item will be removed from the main text
            target_text = target_text.removeprefix(item)
            # treat the item found as a potential candidate
            yield item.strip()
            # return to start of while loop to reconsider the sliced text
        else:
            # no citations found, can adopt the remaining or unsliced text
            # as a candidate
            yield target_text
            break


def use_citation_to_divide(
    line: str,
    source_text: str,
    key: Optional[int] = None,
):
    """ Construct data with citations as dividing mechanism"""
    html = BeautifulSoup(line, "html5lib")
    text = html.get_text()
    for candidate in get_candidates(text):
        if candidate:  # deal with ""
            if (item := create_single_title(candidate, source_text, key)) :
                yield item
