import re
from typing import Dict, Optional

from bs4 import BeautifulSoup, ResultSet, Tag

VS_PATTERN = re.compile(r"\bv\.?s?\.")
RE_PATTERN = re.compile(r"^In\s+(R|r)e:?")
SPECIAL_VS = re.compile(r"<em>\s*v\.?\s*</em>")


def get_indicators(text: str):
    # html indicators
    html = BeautifulSoup(text, "html5lib")
    tagged_vs_simple = html("em", text=VS_PATTERN)
    tagged_inre = html("em", text=RE_PATTERN)

    # non html indicators
    vs_simple = VS_PATTERN.search(text)
    inre = RE_PATTERN.search(text)

    return tagged_vs_simple or tagged_inre or vs_simple or inre


def match_any_versus(text: str):
    return (
        VS_PATTERN.search(text)
        or RE_PATTERN.search(text)
        or SPECIAL_VS.search(text)
    )


def is_single_simple_versus(text: str):
    vs_pattern = VS_PATTERN.findall(text)
    return len(vs_pattern) == 1


def is_two_parties_between_versus(text: str):
    parties = VS_PATTERN.split(text)
    return len(parties) == 2


def one_case_pattern(text: str) -> bool:
    re_list = list(RE_PATTERN.finditer(text))
    vs_list = list(VS_PATTERN.finditer(text))
    vsx_list = list(SPECIAL_VS.finditer(text))
    only_one_vs = len(vs_list + vsx_list) == 1
    only_one_re = len(re_list) == 1
    return only_one_vs or only_one_re


def error_item(
    error_msg: str,
    source: str,
    key: Optional[int] = None,
) -> Dict:
    """
    Is there an error? Create a dictionary object for debugging
    """
    return {
        "raw": source if key else None,
        "key": key,
        "casename": None,
        "citation": None,
        "plaintiff": None,
        "defendant": None,
        "inre": None,
        "error": error_msg,  # could not find valid casename
    }


def ok_count_of_vs_variants_in_tags(tags: ResultSet) -> bool:
    """Special rule for italics on just the vs string. Are all of the vs tags in the text the same variant? e.g. <em>vs.</em> and <em>"et. al. vs"</em> are the same variant."""
    same_vs_variant = [
        "vs",
        "vs.",
        "v",
        "v.",
        "., vs",  # AFP Mutual Benefit Association, Inc<em>., vs. </em>National Labor Relations Commission; Wiltshire File Co. Inc.<em>, v. </em>NLRC
        "et. al. vs",
        "et al. vs",  #  Roselini L. de La Cruz,<em> et al. vs.</em> National Labor Relations Commission,
        "et al., vs",  # Filflex Industrial &amp; Manufacturing Corporation, <em>et al., vs.</em> National Labor Relations Commission
        "et. al. v",  # Philippine Rock Products, Inc. <em>et al. v.</em> PAPLU
        "et al. v",  # # Fernandez <em>et al. v.</em> National Labor Relations Commission
        "et al., v",
    ]
    return len(tags) == len(
        [
            tag
            for tag in tags
            if isinstance(tag, Tag)
            and tag.get_text().strip(",. ") in same_vs_variant
        ]
    )
