import re
from typing import Dict, Iterator, Match, Optional

from bs4 import NavigableString, ResultSet, Tag
from bs4.element import PageElement
from casecite import make_citations

from .split import create_single_title, get_candidates
from .utils import VS_PATTERN, error_item


def is_many_vs_patterns(e: PageElement):
    return len(list(VS_PATTERN.finditer(e.get_text()))) > 1


def case_italics_validator(
    elements: ResultSet,
    source: str,
    key: Optional[int] = None,
) -> Iterator[Dict]:
    for e in elements:

        # many vs in tag
        if is_many_vs_patterns(e):

            for c in get_candidates(e.get_text()):
                # convert text then partition
                if (item := create_single_title(c, source, key)) :
                    yield item

        # only one pattern, e.g <em>X v. Y</em> OR <em>In re: X</em>
        else:

            if (error := red_flags(e)) :
                yield error_item(error, source, key)
                continue

            elif (item := create_single_title(e.get_text(), source, key)) :
                # create single title already creates a citation value
                if item["citation"]:
                    yield item
                    continue

                # no citation existing yet, possible citation in neighboring element
                else:
                    item["citation"] = neighbor(e) or None
                    yield item
                    continue


def neighbor(t: Tag) -> Optional[str]:
    """ if with red flags, can't get citation; else, possible citation in next sibling """
    # check if the indicators do not exist; presence of ibid, supra implies that the casename does not contain a citation
    if (
        tag_ends_ibid(t)
        or tag_includes_supra(t)
        or next_is_string_ibid(t)
        or next_is_string_supra(t)
    ):
        return None
    if not isinstance(t.next_sibling, NavigableString):
        return None
    if not (cites := make_citations(t.next_sibling)):
        return None
    return cites[0]["raw"]


def tag_ends_ibid(t: Tag) -> Optional[Match]:
    # example "<em>AHS/Philippines, Inc. vs. C.A., Ibid</em>., p. 329; Pizza Hut/Progressive Development Corporation vs. National Labor Relations Commission, 252 SCRA 531, 535, January 29, 1996."
    # in this case the first vs. tag is <em> enclosed; the second case is not <em> enclosed
    # without this hack, a citation would be added to AHS
    text = t.get_text()
    ends_in_ibid = re.compile(r"i(bi)?d[.,\s]*$", re.I)
    return ends_in_ibid.search(text)


def tag_includes_supra(t: Tag) -> Optional[Match]:
    text = t.get_text()
    ends_in_supra = re.compile(r"supra[.,\s]*$", re.I)
    return ends_in_supra.search(text)


def next_is_string_ibid(t: Tag) -> Optional[Match]:
    # first element after <em> enclosed is a string and the string's first character sequence is 'SUPRA'
    # [{"key": 37, "value": "<em>People vs. Matrimonio,</em> supra Fn. 33; People vs.Echegaray, G.R. No. 117472, June 25, 1996, 257 SCRA 561.", "raw": "[37]", "case_id": 35558, "citations": [{"raw": "G.R. No. 117472, June 25, 1996, 257 SCRA 561.", "docket": "General Register 117472, June 25, 1996", "phil": null, "scra": "257 SCRA 561", "offg": null, "case_id": 35558}], "statutes": null, "provisions": null, "casenames": [{"raw": "<em>People vs. Matrimonio,</em> supra Fn. 33; People vs.Echegaray, G.R. No. 117472, June 25, 1996, 257 SCRA 561.", "key": 37, "origin": "footnotes", "casename": "People vs. Matrimonio,", "citation": "G.R. No. 117472, June 25, 1996, 257 SCRA 561.", "plaintiff": "People", "defendant": "Matrimonio", "error": null}]}]
    # "<em>People vs. Matrimonio,</em> supra, Fn. 33; <em>People vs. Villanueva, </em>G.R. Nos. 112164-65, February 28, 1996, 254 SCRA 202;"
    if not isinstance(t.next_sibling, NavigableString):
        return None
    text = str(t.next_sibling)
    next_is_ibid = re.compile(r"^[.,\s]+i(bi)?d[.,\s]*$", re.I)
    return next_is_ibid.search(text)


def next_is_string_supra(t: Tag) -> Optional[Match]:
    if isinstance(t.next_sibling, NavigableString):
        return None
    text = str(t.next_sibling)
    next_is_supra = re.compile(r"^[.,\s]+supra[.,\s]*$", re.I)
    return next_is_supra.search(text)


def red_flags(tag: Tag) -> Optional[str]:
    text = tag.get_text()  # format should be: plaintiff vs. defendant
    if len(text) < 8:
        return f"Text too short: {str(tag)}"
    elif len(text) > 150:
        return f"Text too long: {str(tag)}"
    elif text[0] == '"' or text[-1] == '"':
        return f"Text is a casename covered in quotes: {str(tag)}"
    elif text[0] == "“" or text[-1] == "”":
        return f"Text is a casename covered in quotes: {str(tag)}"
    return None
